from django.core.management.base import BaseCommand
from django.contrib.comments.models import Comment
from moderator import models, utils

SAMPLE_COUNT = 10000


class Command(BaseCommand):
    help = 'Reinitializes Baysian inference classifier and trains sample of'\
           'existing comments as ham(if not removed) or spam(if removed).'\
           'WARNING: All previous classification data will be reset(wiped).'

    def handle(self, *args, **options):
        # Clear previous classifications.
        utils.clear()
        models.ClassifiedComment.objects.all().delete()

        # Collect sample of ham and spam comments.
        ham_comments = Comment.objects.filter(
            is_removed=False
        ).order_by('?')[:SAMPLE_COUNT]
        spam_comments = Comment.objects.filter(
            is_removed=True
        ).order_by('?')[:SAMPLE_COUNT]
        comments = list(ham_comments) + list(spam_comments)
        comment_count = len(comments)

        self.stdout.write('Learning from %s comments, please wait...' % comment_count)
        self.stdout.flush()

        perc = 0
        ham = []
        spam = []

        # Train and group ham and spam comments as such.
        for i, comment in enumerate(comments):
            new_perc = int(i / (comment_count / 100.0))
            if new_perc > perc:
                perc = new_perc
                self.stdout.write('.')
                self.stdout.flush()
            if comment.is_removed:
                utils.train(comment, is_spam=True)
                spam.append(models.ClassifiedComment(
                    comment=comment,
                    cls='spam',
                ))
            else:
                utils.train(comment, is_spam=False)
                ham.append(models.ClassifiedComment(
                    comment=comment,
                    cls='ham',
                ))

        # Bulk create classified comment objects for trained objects.
        self.stdout.write('\nCreating classification objects, please wait...\n')
        self.stdout.flush()
        models.ClassifiedComment.objects.bulk_create(ham)
        models.ClassifiedComment.objects.bulk_create(spam)

        self.stdout.write('Done!\n')



from unittest import TestCase

from django.contrib.auth.models import AnonymousUser
from django.contrib.comments.models import Comment
from django.contrib.contenttypes.models import ContentType
from django.core import management
from django.template import Template, Context
from django.test.client import RequestFactory
import fakeredis
from likes.middleware import SecretBallotUserIpUseragentMiddleware
from likes.views import can_vote_test
from moderator.constants import DJANGO_SAMPLE_CONFIG
from moderator.models import ClassifiedComment, ClassifierState, Word
from moderator.storage import DjangoClassifier, RedisClassifier
from secretballot import views
from secretballot.models import Vote


# Before we do anything else monkeypatch FakeRedis into RedisClassifier
RedisClassifier.redis_class = fakeredis.FakeRedis


class BaseClassifierTestCase(object):
    def setUp(self):
        if 'CLASSIFIER_CONFIG' not in self.config:
            self.config['CLASSIFIER_CONFIG'] = {}

        self.classifier = self.classifier_class(
            **self.config['CLASSIFIER_CONFIG']
        )
        self.clear()
        self.classifier = self.classifier_class(
            **self.config['CLASSIFIER_CONFIG']
        )

    def test_init(self):
        # State's counts should be loaded in the classifier.
        # On initial creation of classifier counts are 0.
        self.failUnlessEqual(self.classifier.nspam, 0)
        self.failUnlessEqual(self.classifier.nham, 0)

        # On subsequent load counts are loaded from state.
        self.classifier.nspam = 10
        self.classifier.nham = 20
        self.classifier.store()
        self.classifier = self.classifier_class(
            **self.config['CLASSIFIER_CONFIG']
        )
        self.failUnlessEqual(self.classifier.nspam, 10)
        self.failUnlessEqual(self.classifier.nham, 20)

    def test_store(self):
        state = self.classifier.get_state()
        self.failIfEqual(
            state.spam_count,
            50,
            'Internal checking test is not testing existing values'
        )
        self.failIfEqual(
            state.ham_count,
            100,
            'Internal checking test is not testing existing values'
        )

        # On store classifier counts should be saved to DB.
        self.classifier.nspam = 50
        self.classifier.nham = 100
        self.classifier.store()
        state = self.classifier.get_state()
        self.failUnlessEqual(state.spam_count, 50)
        self.failUnlessEqual(state.ham_count, 100)


class DjangoClassifierTestCase(BaseClassifierTestCase, TestCase):
    classifier_class = DjangoClassifier
    config = DJANGO_SAMPLE_CONFIG

    def clear(self):
        ClassifierState.objects.all().delete()
        Word.objects.all().delete()

    def test_get_state(self):
        # States are unique for type.
        self.classifier = self.classifier_class(
            **self.config['CLASSIFIER_CONFIG']
        )
        self.classifier = self.classifier_class(
            **self.config['CLASSIFIER_CONFIG']
        )
        self.classifier = self.classifier_class(
            **self.config['CLASSIFIER_CONFIG']
        )
        self.classifier = self.classifier_class(
            **self.config['CLASSIFIER_CONFIG']
        )
        self.failUnlessEqual(ClassifierState.objects.all().count(), 1)

        # On initial creation of state counts are 0.
        state = self.classifier.get_state()
        self.failUnlessEqual(state.spam_count, 0)
        self.failUnlessEqual(state.ham_count, 0)

        # On subsequent load counts are loaded from state.
        self.classifier.nspam = 10
        self.classifier.nham = 20
        self.classifier.store()
        state = self.classifier.get_state()
        self.failUnlessEqual(state.spam_count, 10)
        self.failUnlessEqual(state.ham_count, 20)

    def test_wordinfoget(self):
        # Without any existing words should not create Word,
        # just return word_info with zero counts.
        self.failIf(
            Word.objects.all(),
            "Internal check, no words should exist now"
        )
        word_info = self.classifier._wordinfoget('test')
        self.failUnlessEqual(word_info.spamcount, 0)
        self.failUnlessEqual(word_info.hamcount, 0)

        # With existing words should load counts from DB.
        Word.objects.create(word="test", spam_count=10, ham_count=20)
        word_info = self.classifier._wordinfoget('test')
        self.failUnlessEqual(word_info.spamcount, 10)
        self.failUnlessEqual(word_info.hamcount, 20)

    def test_wordinfoset(self):
        # Without any existing words should create Word with count from record.
        self.failIf(
            Word.objects.all(),
            "Internal check, no words should exist now"
        )
        word_info = self.classifier.WordInfoClass()
        word_info.spamcount = 200
        word_info.hamcount = 500
        self.classifier._wordinfoset('test', word_info)
        word = Word.objects.get()
        self.failUnlessEqual(word.word, 'test')
        self.failUnlessEqual(word.spam_count, 200)
        self.failUnlessEqual(word.ham_count, 500)

        # With existing word should update counts from record.
        word_info.spamcount = 222
        word_info.hamcount = 555
        self.classifier._wordinfoset('test', word_info)
        word = Word.objects.get()
        self.failUnlessEqual(word.word, 'test')
        self.failUnlessEqual(word.spam_count, 222)
        self.failUnlessEqual(word.ham_count, 555)


class RedisClassifierTestCase(BaseClassifierTestCase, TestCase):
    classifier_class = RedisClassifier
    config = {'CLASSIFIER_CONFIG': {}}

    def clear(self):
        self.classifier.redis.flushdb()

    def test_get_state(self):
        # On initial creation of state counts are 0.
        state = self.classifier.get_state()
        self.failUnlessEqual(state.spam_count, 0)
        self.failUnlessEqual(state.ham_count, 0)

        # On subsequent load counts are loaded from state.
        self.classifier.nspam = 10
        self.classifier.nham = 20
        self.classifier.store()
        state = self.classifier.get_state()
        self.failUnlessEqual(state.spam_count, 10)
        self.failUnlessEqual(state.ham_count, 20)

    def test_wordinfoget(self):
        # Without any existing words should not create Word,
        # just return word_info with zero counts.
        word_info = self.classifier._wordinfoget('test')
        self.failUnlessEqual(word_info.spamcount, 0)
        self.failUnlessEqual(word_info.hamcount, 0)

        # With existing words should load counts from Redis.
        self.classifier.redis.set('test_spam_count', '10')
        self.classifier.redis.set('test_ham_count', '20')
        word_info = self.classifier._wordinfoget('test')
        self.failUnlessEqual(word_info.spamcount, 10)
        self.failUnlessEqual(word_info.hamcount, 20)

    def test_wordinfoset(self):
        # Without any existing words should create Word with count from record.
        word_info = self.classifier.WordInfoClass()
        word_info.spamcount = 200
        word_info.hamcount = 500
        self.classifier._wordinfoset('test', word_info)

        word_info = self.classifier._wordinfoget('test')
        self.failUnlessEqual(word_info.spamcount, 200)
        self.failUnlessEqual(word_info.hamcount, 500)

        # With existing word should update counts from record.
        word_info.spamcount = 222
        word_info.hamcount = 555
        self.classifier._wordinfoset('test', word_info)
        word_info = self.classifier._wordinfoget('test')
        self.failUnlessEqual(word_info.spamcount, 222)
        self.failUnlessEqual(word_info.hamcount, 555)


class ManagementCommandTestCase(TestCase):
    def setUp(self):
        from moderator import utils
        from moderator.classifier import classifier
        self.utils = utils
        # Reset words and counts
        classifier.bayes.redis.flushdb()
        classifier.bayes.nspam = 0
        classifier.bayes.nham = 0
        classifier.store()
        Word.objects.all().delete()
        ClassifiedComment.objects.all().delete()
        Comment.objects.all().delete()

    def test_classifycomments(self):

        # Initial comment setup.
        spam_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="very bad spam"
        )
        ham_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="awesome tasty ham"
        )
        unsure_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="awesome spam"
        )
        untrained_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="foo bar"
        )

        # Initial training
        self.utils.train(spam_comment, True)
        self.utils.train(ham_comment, False)

        management.call_command('classifycomments')

        self.failUnlessEqual(ClassifiedComment.objects.all().count(), 4,
                             "We should now have 4 classified comments")

        # Comment previously trained as spam should be classed as spam.
        ClassifiedComment.objects.get(comment=spam_comment, cls='spam')

        # Comment previously trained as ham should be classed as ham.
        ClassifiedComment.objects.get(comment=ham_comment, cls='ham')

        # Comment containing words previously trained as both
        # spam and ham should be classed as unsure.
        ClassifiedComment.objects.get(comment=unsure_comment, cls='unsure')

        # Comment containing words not previously trained should be
        # classed as unsure.
        ClassifiedComment.objects.get(comment=untrained_comment, cls='unsure')


class UtilsTestCase(TestCase):
    def setUp(self):
        from moderator import utils
        from moderator.classifier import classifier
        self.utils = utils
        self.classifier = classifier
        # Reset words and counts to zero.
        self.classifier.bayes.redis.flushdb()
        self.classifier.bayes.nspam = 0
        self.classifier.bayes.nham = 0
        self.classifier.store()

    def test_train(self):
        self.failUnlessEqual(self.classifier.bayes.nham, 0,
                             "No ham has been trained yet, should be 0")
        self.failUnlessEqual(self.classifier.bayes.nspam, 0,
                             "No spam has been trained yet, should be 0")

        comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="very bad spam"
        )
        self.utils.train(comment, True)

        # Now we should have word info for each word in the comment,
        # each having spam count of 1 and ham count of 0.
        word_info = self.classifier.bayes._wordinfoget('very')
        self.failUnlessEqual(word_info.spamcount, 1)
        self.failUnlessEqual(word_info.hamcount, 0)
        word_info = self.classifier.bayes._wordinfoget('bad')
        self.failUnlessEqual(word_info.spamcount, 1)
        self.failUnlessEqual(word_info.hamcount, 0)
        word_info = self.classifier.bayes._wordinfoget('spam')
        self.failUnlessEqual(word_info.spamcount, 1)
        self.failUnlessEqual(word_info.hamcount, 0)

        self.failUnlessEqual(self.classifier.bayes.nham, 0,
                             "No ham has been trained yet, should still be 0")
        self.failUnlessEqual(self.classifier.bayes.nspam, 1,
                             "Spam has been trained, should be 1")

        comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="awesome tasty ham"
        )
        self.utils.train(comment, False)

        # Now we should have word info for each word in the comment,
        # each having ham count of 1 and spam count of 0.
        word_info = self.classifier.bayes._wordinfoget('awesome')
        self.failUnlessEqual(word_info.spamcount, 0)
        self.failUnlessEqual(word_info.hamcount, 1)
        word_info = self.classifier.bayes._wordinfoget('tasty')
        self.failUnlessEqual(word_info.spamcount, 0)
        self.failUnlessEqual(word_info.hamcount, 1)
        word_info = self.classifier.bayes._wordinfoget('ham')
        self.failUnlessEqual(word_info.spamcount, 0)
        self.failUnlessEqual(word_info.hamcount, 1)

        self.failUnlessEqual(
            self.classifier.bayes.nham,
            1,
            "Ham has been trained, should still 1"
        )
        self.failUnlessEqual(
            self.classifier.bayes.nspam,
            1,
            "No more spam has been trained, should still be 1"
        )

        # Training should store state
        state = self.classifier.bayes.get_state()
        self.failUnlessEqual(state.spam_count, 1)
        self.failUnlessEqual(state.ham_count, 1)

    def test_get_class(self):
        # Initial comment setup.
        spam_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="very bad spam"
        )
        ham_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="awesome tasty ham"
        )
        unsure_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="awesome spam"
        )
        untrained_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="foo bar"
        )

        # Initial training
        self.utils.train(spam_comment, True)
        self.utils.train(ham_comment, False)

        # Comment previously trained as spam should be classed as spam.
        self.failUnlessEqual(self.utils.get_class(spam_comment), 'spam')

        # Comment previously trained as ham should be classed as ham.
        self.failUnlessEqual(self.utils.get_class(ham_comment), 'ham')

        # Comment containing words previously trained as both
        # spam and ham should be classed as unsure.
        self.failUnlessEqual(self.utils.get_class(unsure_comment), 'unsure')

        # Comment containing words not previously trained should be
        # classed as unsure.
        self.failUnlessEqual(self.utils.get_class(untrained_comment), 'unsure')

    def test_classify_comment(self):
        spam_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="very bad spam"
        )
        ham_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="awesome tasty ham"
        )
        unsure_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="tasty spam"
        )
        generic_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="foo bar"
        )
        abusive_comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="abusive comment"
        )
        for i in range(0, 3):
            Vote.objects.create(
                content_type=ContentType.objects.get_for_model(Comment),
                object_id=abusive_comment.id,
                token=i,
                vote=-1
            )

        # Providing unsure should create unsure classification without
        # training.
        classified_comment = self.utils.classify_comment(
            generic_comment,
            'unsure'
        )
        self.failIf(self.classifier.bayes.nspam)
        self.failIf(self.classifier.bayes.nham)
        self.failUnlessEqual(classified_comment.cls, 'unsure')

        # Providing reported should create reported classification without
        # training, comment should be removed.
        classified_comment = self.utils.classify_comment(
            generic_comment,
            'reported'
        )
        self.failIf(self.classifier.bayes.nspam)
        self.failIf(self.classifier.bayes.nham)
        self.failUnlessEqual(classified_comment.cls, 'reported')
        self.failUnless(classified_comment.comment.is_removed)

        # Without providing a class but with user abuse reports more or equal
        # to cutoff should create reported classification without training,
        # comment should be removed.
        classified_comment = self.utils.classify_comment(abusive_comment)
        self.failUnlessEqual(self.classifier.bayes.nspam, 0)
        self.failIf(self.classifier.bayes.nham)
        self.failUnlessEqual(classified_comment.cls, 'reported')
        self.failUnless(classified_comment.comment.is_removed)

        # Providing ham class should create ham classification and training,
        # comment should not be removed.
        classified_comment = self.utils.classify_comment(ham_comment, 'ham')
        self.failUnlessEqual(self.classifier.bayes.nspam, 0)
        self.failUnlessEqual(self.classifier.bayes.nham, 1)
        self.failUnlessEqual(classified_comment.cls, 'ham')
        self.failIf(classified_comment.comment.is_removed)

        # Providing spam class should create spam classification and training,
        # comment should be removed.
        classified_comment = self.utils.classify_comment(spam_comment, 'spam')
        self.failUnlessEqual(self.classifier.bayes.nspam, 1)
        self.failUnlessEqual(self.classifier.bayes.nham, 1)
        self.failUnlessEqual(classified_comment.cls, 'spam')
        self.failUnless(classified_comment.comment.is_removed)

        # Spammy comment should now be correctly classified automatically
        # without any training, should be removed.
        comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="bad spam"
        )
        comment.total_downvotes = 0
        classified_comment = self.utils.classify_comment(comment)
        self.failUnlessEqual(self.classifier.bayes.nspam, 1)
        self.failUnlessEqual(self.classifier.bayes.nham, 1)
        self.failUnlessEqual(classified_comment.cls, 'spam')
        self.failUnless(classified_comment.comment.is_removed)

        # Hammy comment should now be correctly classified automatically
        # without any training, should not be removed.
        comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="tasty ham"
        )
        comment.total_downvotes = 0
        classified_comment = self.utils.classify_comment(comment)
        self.failUnlessEqual(self.classifier.bayes.nspam, 1)
        self.failUnlessEqual(self.classifier.bayes.nham, 1)
        self.failUnlessEqual(classified_comment.cls, 'ham')
        self.failIf(classified_comment.comment.is_removed)

        # Hammy spammy comment should now be correctly classified automatically
        # as unsure without any training, should not be removed.
        classified_comment = self.utils.classify_comment(unsure_comment)
        self.failUnlessEqual(self.classifier.bayes.nspam, 1)
        self.failUnlessEqual(self.classifier.bayes.nham, 1)
        self.failUnlessEqual(classified_comment.cls, 'unsure')
        self.failIf(classified_comment.comment.is_removed)

        # Should raise exception with unkown cls.
        self.assertRaises(Exception, self.utils.classify_comment,
                          unsure_comment, 'unknown_cls')


class InclusionTagsTestCase(TestCase):

    def test_report_comment_abuse(self):
        # Prepare context.
        context = Context()
        request = RequestFactory().get('/')
        request.user = AnonymousUser()
        request.META['HTTP_USER_AGENT'] = 'testing_agent'
        request.secretballot_token = SecretBallotUserIpUseragentMiddleware().\
            generate_token(request)
        comment = Comment.objects.create(
            content_type_id=1,
            site_id=1,
            comment="abuse report testing comment"
        )
        context['request'] = request
        context['comment'] = comment

        # Without having actioned anything on the comment the
        # Report Abuse action should be rendered.
        out = Template("{% load moderator_inclusion_tags %}"
                       "{% report_comment_abuse comment %}").render(context)
        self.failUnless('Report Abuse' in out)

        # Like a comment.
        views.vote(
            request,
            content_type='.'.join((comment._meta.app_label,
                                   comment._meta.module_name)),
            object_id=comment.id,
            vote=1,
            redirect_url='/',
            can_vote_test=can_vote_test
        )

        # With having liked a comment nothing should be rendered.
        out = Template("{% load moderator_inclusion_tags %}"
                       "{% report_comment_abuse comment %}").render(context)
        self.failUnlessEqual(out, '\n')

        # Reset previous like and test it applied.
        Vote.objects.all().delete()

        # Without having actioned anything on the comment the
        # Report Abuse action should be rendered.
        out = Template("{% load moderator_inclusion_tags %}"
                       "{% report_comment_abuse comment %}").render(context)
        self.failUnless('Report Abuse' in out)

        # Dislike/report an abuse comment.
        views.vote(
            request,
            content_type='.'.join((comment._meta.app_label,
                                   comment._meta.module_name)),
            object_id=comment.id,
            vote=-1,
            redirect_url='/',
            can_vote_test=can_vote_test
        )

        # With having reported abuse an acknowledgement should be rendered.
        out = Template("{% load moderator_inclusion_tags %}"
                       "{% report_comment_abuse comment %}").render(context)
        self.failUnless('Abuse Reported' in out)



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'CannedReply.site'
        db.add_column('moderator_cannedreply', 'site',
                      self.gf('django.db.models.fields.related.ForeignKey')(to=orm['sites.Site'], null=True, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'CannedReply.site'
        db.delete_column('moderator_cannedreply', 'site_id')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'comments.comment': {
            'Meta': {'ordering': "('submit_date',)", 'object_name': 'Comment', 'db_table': "'django_comments'"},
            'comment': ('django.db.models.fields.TextField', [], {'max_length': '3000'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'content_type_set_for_comment'", 'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip_address': ('django.db.models.fields.IPAddressField', [], {'max_length': '15', 'null': 'True', 'blank': 'True'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_removed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'object_pk': ('django.db.models.fields.TextField', [], {}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']"}),
            'submit_date': ('django.db.models.fields.DateTimeField', [], {'default': 'None'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'comment_comments'", 'null': 'True', 'to': "orm['auth.User']"}),
            'user_email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'user_name': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'user_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'moderator.cannedreply': {
            'Meta': {'object_name': 'CannedReply'},
            'comment': ('django.db.models.fields.TextField', [], {'max_length': '3000'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'site': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['sites.Site']", 'null': 'True', 'blank': 'True'})
        },
        'moderator.classifiedcomment': {
            'Meta': {'ordering': "['-comment__submit_date']", 'object_name': 'ClassifiedComment'},
            'cls': ('django.db.models.fields.CharField', [], {'max_length': '64'}),
            'comment': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['comments.Comment']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'moderator.classifierstate': {
            'Meta': {'object_name': 'ClassifierState'},
            'ham_count': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'spam_count': ('django.db.models.fields.IntegerField', [], {})
        },
        'moderator.commentreply': {
            'Meta': {'object_name': 'CommentReply'},
            'canned_reply': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['moderator.CannedReply']", 'null': 'True', 'blank': 'True'}),
            'comment': ('django.db.models.fields.TextField', [], {'max_length': '3000', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'replied_to_comment': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'replied_to_comment_set'", 'to': "orm['comments.Comment']"}),
            'reply_comment': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'reply_comment_set'", 'to': "orm['comments.Comment']"}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'moderator.word': {
            'Meta': {'object_name': 'Word'},
            'ham_count': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'spam_count': ('django.db.models.fields.IntegerField', [], {}),
            'word': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'secretballot.vote': {
            'Meta': {'unique_together': "(('token', 'content_type', 'object_id'),)", 'object_name': 'Vote'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'token': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'vote': ('django.db.models.fields.SmallIntegerField', [], {})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['moderator']


from django import template
from django.conf import settings
from likes.utils import can_vote
from moderator.constants import DEFAULT_CONFIG
from secretballot.models import Vote

register = template.Library()


@register.inclusion_tag('moderator/inclusion_tags/report_comment_abuse.html',
                        takes_context=True)
def report_comment_abuse(context, obj):
    """
    Checks whether a user can report abuse (has not liked comment previously)
    or has reported abuse previously and renders appropriate response.

    If requesting user is part of the 'Moderators' group a vote equal to
    ABUSE_CUTOFF setting will be made, thereby immediately marking the comment
    as abusive.
    """
    request = context['request']
    is_moderator = bool(request.user.groups.filter(name='Moderators'))
    abuse_cutoff = getattr(settings, 'MODERATOR', DEFAULT_CONFIG)['ABUSE_CUTOFF']

    can_report = can_vote(obj, request.user, request)
    previously_reported = False
    if not can_report:
        previously_reported = Vote.objects.filter(
            object_id=obj.id,
            token=request.secretballot_token,
            vote__in=['-1', '-%s' % abuse_cutoff]
        ).count() != 0

    context.update({
        'content_obj': obj,
        'previously_reported': previously_reported,
        'can_report': can_report,
        'vote': (-1 * abuse_cutoff) if is_moderator else -1,
        'content_type': "-".join((obj._meta.app_label, obj._meta.module_name)),
    })
    return context



from spambayes.hammie import Hammie
from django.conf import settings
from moderator.constants import DEFAULT_CONFIG

# Get conf from settings or fallback to default.
config = getattr(settings, 'MODERATOR', DEFAULT_CONFIG)

# Load classifier class from config string.
parts = config['CLASSIFIER'].split('.')
module_name = '.'.join(parts[:-1])
class_name = parts[-1]
mod = __import__(module_name, fromlist=[class_name, ])
bayes_class = getattr(mod, class_name)

# Create classifier using determined bayes class
# object instantiated with config.
if 'CLASSIFIER_CONFIG' in config:
    classifier_config = config['CLASSIFIER_CONFIG']
else:
    classifier_config = {}
classifier = Hammie(bayes=bayes_class(**classifier_config), mode='w')



from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from moderator.classifier import classifier
from moderator.constants import DEFAULT_CONFIG
from secretballot.models import Vote
from moderator import models


def train(comment, is_spam):
    classifier.train(comment.comment, is_spam)
    classifier.store()


def clear():
    classifier.bayes.clear()


def get_class(comment):
    """
    Returns comment class as determined by Baysian classifier.
    """
    try:
        score = classifier.score(comment.comment)
    except ZeroDivisionError:
        return 'unsure'

    if score < getattr(settings, 'MODERATOR', DEFAULT_CONFIG)['HAM_CUTOFF']:
        return 'ham'
    if score > getattr(settings, 'MODERATOR', DEFAULT_CONFIG)['SPAM_CUTOFF']:
        return 'spam'
    return 'unsure'


def classify_comment(comment, cls=None):
    """
    Trains Baysian inference classifier with comment and provided class
    (either 'ham' or 'spam').

    If 'unsure' class is provided no training occures, the comment's class
    is simply set as such.

    If 'reported' class is provided no training occures, the comment's class
    is simply set as such and removed.

    If no class is provided a lookup is done to see if the comment has been
    reported by users as abusive. If indicated as abusive class is set
    as 'reported', with spam training occuring and the comment being removed.

    If a comment is not reported as abusive by users and without a class being
    provided a class is determined using Baysian inference. In this case no
    training occurs to prevent self learning.

    Returns a newly created or updated ClassifiedComment object.
    As a side effect also sets is_removed field of comment based on class.
    """
    if cls not in ['spam', 'ham', 'unsure', 'reported', None]:
        raise Exception("Unrecognized classifications.")

    classified_comment, created = models.ClassifiedComment.objects.get_or_create(
        comment=comment
    )

    if cls == 'spam' and classified_comment.cls != 'spam':
        train(comment, is_spam=True)
        comment.is_removed = True
        comment.save()
        classified_comment.cls = cls
        classified_comment.save()
        return classified_comment

    if cls == 'ham' and classified_comment.cls != 'ham':
        train(comment, is_spam=False)
        comment.is_removed = False
        comment.save()
        classified_comment.cls = cls
        classified_comment.save()
        return classified_comment

    if cls == 'unsure' and classified_comment.cls != 'unsure':
        classified_comment.cls = cls
        classified_comment.save()
        return classified_comment

    if cls == 'reported' and classified_comment.cls != 'reported':
        train(comment, is_spam=True)
        comment.is_removed = True
        comment.save()
        classified_comment.cls = cls
        classified_comment.save()
        return classified_comment

    if cls is None:
        comment_content_type = ContentType.objects.get_for_model(comment)
        moderator_settings = getattr(settings, 'MODERATOR', DEFAULT_CONFIG)
        if Vote.objects.filter(
            content_type=comment_content_type,
            object_id=comment.id,
            vote=-1
        ).count() >= moderator_settings['ABUSE_CUTOFF']:
            cls = 'reported'
            comment.is_removed = True
            comment.save()
            classified_comment.cls = cls
            classified_comment.save()
            return classified_comment
        else:
            cls = get_class(comment)
            comment.is_removed = cls == 'spam'
            comment.save()
            classified_comment.cls = cls
            classified_comment.save()
            return classified_comment

    return classified_comment



from optparse import make_option

from django.core.management.base import BaseCommand
from django.contrib.comments.models import Comment
from django.db.models import Q
from moderator import utils


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('-c', '--count',
            dest='count',
            type="int",
            default=0,
            help='Number of comments to classify.'),
        )
    help = 'Classifies comments as either spam or '\
           'ham using Bayesian inference and user reports.'

    def handle(self, *args, **options):
        # Collect all comments that hasn't already been classified or are classified as unsure.
        # Order randomly so we don't rehash previously unsure classifieds when count limiting.
        comments = Comment.objects.filter(Q(classifiedcomment__isnull=True) | Q(classifiedcomment__cls='unsure')).order_by('?')
        if options['count']:
            comments = comments[:options['count']]
        comment_count = comments.count()

        self.stdout.write('Classifying %s comments, please wait...' % comment_count)
        self.stdout.flush()

        for comment in comments:
            classified_comment = utils.classify_comment(comment)
            self.stdout.write('%s,' % classified_comment.cls[0])
            self.stdout.flush()

        self.stdout.write('\nDone!\n')



from moderator.models import ClassifierState, Word
import redis
from spambayes.classifier import Classifier


class DjangoClassifier(Classifier):
    def __init__(self):
        Classifier.__init__(self)

        # Set state from DB stored value.
        state = self.get_state()
        self.nspam = state.spam_count
        self.nham = state.ham_count

    def clear(self):
        ClassifierState.objects.all().delete()
        Word.objects.all().delete()
        state = self.get_state()
        self.nspam = state.spam_count
        self.nham = state.ham_count

    def get_state(self):
        """
        Retrieves classifier state from DB.
        """
        try:
            return ClassifierState.objects.get()
        except ClassifierState.DoesNotExist:
            return ClassifierState.objects.create(spam_count=0, ham_count=0)

    def store(self):
        """
        Stores classifier state in DB.
        """
        state = self.get_state()
        state.spam_count = self.nspam
        state.ham_count = self.nham
        state.save()

    def _wordinfoget(self, word):
        """
        Get word info from DB.
        If no word exists in the DB use zero values.
        """
        word_info = self.WordInfoClass()
        try:
            word_obj = Word.objects.get(word=word)
            word_info = self.WordInfoClass()
            word_info.__setstate__((word_obj.spam_count, word_obj.ham_count))
        except Word.DoesNotExist:
            pass
        return word_info

    def _wordinfoset(self, word, record):
        """
        Save word info to DB.
        If no word exists it is created with record counts.
        """
        try:
            word_obj = Word.objects.get(word=word)
            word_obj.spam_count = record.spamcount
            word_obj.ham_count = record.hamcount
            word_obj.save()
        except Word.DoesNotExist:
            Word.objects.create(
                word=word,
                spam_count=record.spamcount,
                ham_count=record.hamcount
            )


class RedisClassifier(Classifier):
    redis_class = redis.StrictRedis
    state_keys = {
        'spam': 'redis_classifier_spam_state_key',
        'ham': 'redis_classifier_ham_state_key',
    }

    def __init__(self, *args, **kwargs):
        Classifier.__init__(self)

        self.redis = self.redis_class(**kwargs)
        # Set state from Redis stored value.
        state = self.get_state()
        self.nspam = state.spam_count
        self.nham = state.ham_count

    def clear(self):
        self.redis.flushdb()
        state = self.get_state()
        self.nspam = state.spam_count
        self.nham = state.ham_count

    def get_state(self):
        """
        Retrieves classifier state from Redis.
        """
        class State(object):
            def __init__(self, spam_count, ham_count):
                self.spam_count = spam_count
                self.ham_count = ham_count

        spam = self.redis.get(self.state_keys['spam'])
        ham = self.redis.get(self.state_keys['ham'])
        if spam is None:
            self.redis.set(self.state_keys['spam'], '0')
            spam = self.redis.get(self.state_keys['spam'])
        if ham is None:
            self.redis.set(self.state_keys['ham'], '0')
            ham = self.redis.get(self.state_keys['ham'])

        return State(spam_count=int(spam), ham_count=int(ham))

    def store(self):
        """
        Stores classifier state in Redis.
        """
        self.redis.set(self.state_keys['spam'], str(self.nspam))
        self.redis.set(self.state_keys['ham'], str(self.nham))

    def _wordinfoget(self, word):
        """
        Get word info from Redis.
        If no word exists in the Redis use zero values.
        Spam and ham counts are stored using Redis key formed by appending
        appended '_spam_count' and '_ham_count' to word in question.
        """
        spam_count = self.redis.get('%s_spam_count' % word)
        ham_count = self.redis.get('%s_ham_count' % word)
        if spam_count is None:
            spam_count = 0
        if ham_count is None:
            ham_count = 0

        word_info = self.WordInfoClass()
        word_info.__setstate__((int(spam_count), int(ham_count)))
        return word_info

    def _wordinfoset(self, word, record):
        """
        Save word info to Redis.
        If no word exists it is created with record counts.
        """
        self.redis.set('%s_spam_count' % word, record.spamcount)
        self.redis.set('%s_ham_count' % word, record.hamcount)



DJANGO_SAMPLE_CONFIG = {
    'CLASSIFIER': 'moderator.storage.DjangoClassifier',
    'HAM_CUTOFF': 0.3,
    'SPAM_CUTOFF': 0.7,
    'ABUSE_CUTOFF': 3,
}

REDIS_SAMPLE_CONFIG = {
    'CLASSIFIER': 'moderator.storage.RedisClassifier',
    'CLASSIFIER_CONFIG': {
        'host': 'localhost',
        'port': 6379,
        'db': 0,
        'password': None,
        'socket_timeout': None,
        'connection_pool': None,
        'charset': 'utf-8',
        'errors': 'strict',
        'unix_socket_path': None,
    },
    'HAM_CUTOFF': 0.3,
    'SPAM_CUTOFF': 0.7,
    'ABUSE_CUTOFF': 3,
}

DEFAULT_CONFIG = DJANGO_SAMPLE_CONFIG

CLASS_CHOICES = (
    ('reported', 'Reported'),
    ('spam', 'Spam'),
    ('ham', 'Ham'),
    ('unsure', 'Unsure'),
)
